%* - - - - - - - ;
%* Macro ODSauto ; %* Vs. 0.0.1, by Jim Groeneveld, 23 Oct 2013;
%* - - - - - - - ;

%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Strategy:                                                                  *;
%* =========                                                                  *;
%* Macro ODSauto checks for existence of opened ODS destinations and returns  *;
%* them as a list. Under SAS 9.2 (and up) this can be done by processing the  *;
%* contents of dataset SASHELP.VDEST, but under SAS 9.1.x that dataset does   *;
%* not exist. In that case a workaround is to process the contents of SASHELP.*;
%* VEXTFL that shows the opened files by SAS and by the user. ODS destination *;
%* files are apparently opened by SAS and can be recognised by their Fileref  *;
%* that starts with '#LN'. The physical file paths, names and extensions are  *;
%* also available and from the extensions it can be seen what type of file it *;
%* concerns. It is assumed that the extensions correspond to the already      *;
%* opened ODS destinations, this is 99.9% true. The HTML destination is       *;
%* determined from both the .HTML and the .HTM ext, LISTING from the .LST ext.*;
%*                                                                            *;
%* The interpretation is quite reliable, optimum reachable with SAS 9.1. This *;
%* strategy should/might work with SAS vs. 9.2 as well (I can`t check that).  *;
%* However, the .LST extension is always found, even if ODS closed. Hence     *;
%* checking the LISTING destination has been disabled with SAS version 9.1.x. *;
%*                                                                            *;
%* However, SAS vs. 9.2 (and up) offers new ODS features that enable exact    *;
%* and certain determination of opened ODS destinations (SASHELP.VDEST).      *;
%* The macro ODSauto.sas determines specific ODS destinations (PDS, HTML,     *;
%* RTF, PS and LISTING) that way, dependent on the SAS version (9.1 <> rest). *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of ODSauto:                                                        *;
%* ====================                                                       *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0.0    28 Jul 2010  1st work release supporting SAS vs. 9.1 only         *;
%* 0.0.1    23 Oct 2013  added support for SAS vs. 9.2 and 9.3 (9.x?)         *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;

%MACRO ODSauto (Returned=);
%* Macro variable to store Returned value must be present;

%* Check;
  %IF (%UPCASE(&Returned) EQ RETURNED) %THEN %DO;
    %PUT *** ODSauto *** WARNING: Returned argument may not be 'Returned', can not return this!;
    %GOTO Finish;
  %END;
  %ELSE %IF (NOT %LENGTH(%SUPERQ(Returned))) %THEN %DO; %* empty, unspecified;
    %PUT *** ODSauto *** WARNING: Returned argument may not be empty, specify returned argument!;
    %GOTO Finish;
  %END;
  %ELSE %IF (%UPCASE(&Returned) NE ODS_LIST) %THEN %DO; %* anything else than ODS_LIST;
    %LOCAL ODS_list /* to return */;
  %END;
%* else: ODS_LIST: no action, may be global or local variable at calling level;

%* Core;
  %LET ODS_list = ; %* initialise empty;

  %IF (&SYSVER EQ 9.1) %THEN %DO;
    %LOCAL I /* counter */
    File_Ext /* found file extension */
    EXT_list /* given, extendable */
    ;
%* file extension .LST always found, even if ODS LISTING CLOSE;
    %LET EXT_list = /*LST*/ PDF RTF HTML HTM PS; %* and whatever to add;

    DATA _NULL_;
      LENGTH File_Ext $ 8;
      SET SASHELP.VEXTFL (WHERE=(Fileref EQ: '#LN'));
      %LET I = 1;
      %LET File_Ext = %SCAN ( &EXT_list, &I, %STR( ) );
      %DO %WHILE (%LENGTH(&File_Ext));
        IF ( UPCASE ( SCAN ( xpath, -1, '.' ) ) EQ "&File_Ext" ) THEN DO;
          File_Ext = "&File_Ext";
          IF (File_Ext EQ 'LST') THEN File_Ext = 'LISTING';
          ELSE IF (File_Ext EQ 'HTM') THEN File_Ext = 'HTML';
          CALL EXECUTE ('%LET ODS_list = &ODS_list ' || TRIM(File_Ext) || ';');
        END;
        %LET I = %EVAL ( &I + 1 );
        %LET File_Ext = %SCAN ( &EXT_list, &I, %STR( ) );
      %END;
    RUN;
  %END; %* SAS vs 9.1;

  %ELSE %DO; %* SAS vs 9.2 + up;
    DATA _NULL_;
      SET SASHELP.VDEST; %* Destination: (0 or more of) LISTING PDF HTML RTF PS;
      CALL EXECUTE ('%LET ODS_list = &ODS_list ' || TRIM(Destination) || ';');
    RUN;
  %END; %* SAS vs 9.2 + up;

  %LET &Returned = &ODS_list; %* (no return value because of SAS code);

%Finish:

%* - - - - - ;
%MEND ODSauto;
%* - - - - - ;
